#!/bin/bash

DIR=`dirname $0`/..

GENERATOR=$DIR/hordedoc/docutils/html.py
CONFIG=$DIR/hordedoc/docutils/docutils.conf
TARGET=$DIR/hordeweb
BLACKLIST="CHANGES RELEASE_NOTES"

# Check if everything is in place
test -x $GENERATOR || {
    echo html.py not found.
    exit
}
test -f $CONFIG || {
    echo docutils.conf not found.
    exit
}
test -d $TARGET || {
    echo $TARGET not found.
    exit
}

echo > $TARGET/errors.txt

# Search for docs directories
echo "Searching applications..."
DOC_DIRS=`find $DIR -name docs -a -type d`
echo "Generating documentation..."
for DOC_DIR in $DOC_DIRS; do
    APP=`basename \`dirname $DOC_DIR\``
    if [ $APP != ".." ] &&
        [ -f $DOC_DIR/../README ] &&
        [ "$1" = "" -o "$1" = "$APP" ]; then
        echo -n "$APP "
        mkdir -p $TARGET/$APP/docs
        # Generate HTML docs
        DOCS=$TARGET/$APP/docs/docs.html
        FILES=`find $DOC_DIR -type f -maxdepth 1 -regex .*/[A-Z_]+ | sort`
        FILES="$DOC_DIR/../README $FILES"
        #if [ $APP = "horde" ]; then FILES="$FILES $DOC_DIR/../po/README"; fi
        cat > $DOCS <<EOF
<h3>Documentation</h3>

<p>These are the documentation files as distributed with the application's
tarballs and CVS checkouts.</p>

<ul>
EOF

        for FILE in $FILES; do
            for BLACK in $BLACKLIST; do
                if [ $BLACK = `basename $FILE` ]; then continue 2; fi
            done
            echo -n .
            TO=`basename $FILE .txt`.html
            OUTPUT=`$GENERATOR \
                --generator \
                --source-url="http://cvs.horde.org/co.php/$APP/docs/"\`basename $FILE\` \
                --output-encoding=ISO-8859-1 \
                --rfc-references \
                $FILE \
                2>>$TARGET/errors.txt`
            if [ $? ]; then
                OUTPUT=`echo "$OUTPUT" | sed '1,/<body>/d' | sed '/<\/body>/,$d'`;
                echo "$OUTPUT" > $TARGET/$APP/docs/$TO;
                DISPLAY=`echo $FILE | sed "s|$DOC_DIR/\(../\)\?||"`
                echo "<li><a href=\"?f=$TO\">$DISPLAY</a></li>" >> $DOCS
            fi
        done
        echo "</ul>" >> $DOCS
        echo " done"
    fi
done
